@@ -7,7 +7,7 @@
  * as published by the Free Software Foundation; either version 2, or (at your
  * option) any later version.
  *
- *	$Id: rate_engine.c,v 1.24 2004/01/22 15:04:02 tholo Exp $
+ *	$Id: rate_engine.c,v 1.26 2004/08/26 16:50:18 tholo Exp $
  */
 
 #ifdef __linux__
@@ -34,6 +34,7 @@
 #include <asterisk/options.h>
 #include <asterisk/cli.h>
 #include <asterisk/manager.h>
+#include <asterisk/utils.h>
 
 static char *tdesc = "Call Routing and Rating Application";
 static char *cdr_name = "ratecall";
@@ -57,7 +58,8 @@ STANDARD_LOCAL_USER;
 
 LOCAL_USER_DECL;
 
-static ast_mutex_t ratelock = AST_MUTEX_INITIALIZER;
+AST_MUTEX_DEFINE_STATIC(ratelock);
+
 static MYSQL mysql;
 static char *systemname;
 
@@ -178,7 +180,7 @@ static int routecall_exec(struct ast_channel *chan, void *data)
 {
     char destvar[20], firstvar[20], restvar[20], startcost[20], periodcost[20];
     char trialcost[20];
-    char *args, *number, *flags, *destination;
+    char *args, *number, *flags, *destination, *sub;
     int res = 0, maxcall = 0, i, cmpval;
     struct localuser *u;
     char tempstr[16];
@@ -330,7 +332,19 @@ static int routecall_exec(struct ast_channel *chan, void *data)
 	snprintf(periodcost, sizeof(periodcost), "PERIODCOST%d", i + 1);
 	snprintf(trialcost, sizeof(trialcost), "TRIALCOST%d", i + 1);
 
-	destination = pcre_subst(val->rate->egress->pcre, val->rate->egress->extra, number, strlen(number), 0, 0, val->rate->egress->substitute);
+	sub = malloc(strlen(val->rate->egress->technology) +
+		     strlen(val->rate->egress->peer) +
+		     strlen(val->rate->egress->substitute) +
+		     3);	/* Two /'s and a NUL */
+	strcpy(sub, val->rate->egress->technology);
+	strcat(sub, "/");
+	strcat(sub, val->rate->egress->peer);
+	strcat(sub, "/");
+	strcat(sub, val->rate->egress->substitute);
+	destination = pcre_subst(val->rate->egress->pcre, val->rate->egress->extra, number, strlen(number), 0, 0, sub);
+	if (sub)
+	    free(sub);
+
 	res = 0;
 	pbx_builtin_setvar_helper(chan, destvar, destination);
 	snprintf(tempstr, sizeof(tempstr), "%d", val->rate->firstperiod);
@@ -447,8 +461,11 @@ static int cdr_ratecall(struct ast_cdr *cdr)
      * will allow us to retrieve channel variables and other info
      * that is needed for accurate logging
      */
-    for (chan = ast_channel_walk(NULL); chan && chan->cdr != cdr; chan = ast_channel_walk(chan))
-	;
+    chan = ast_channel_walk_locked(NULL);
+    while (chan && chan->cdr != cdr) {
+	ast_mutex_unlock(&chan->lock);
+	chan = ast_channel_walk_locked(chan);
+    }
     if (chan) {
 	for (i = 1;; i++) {
 	    snprintf(destvar, sizeof(destvar), "DESTINATION%d", i);
@@ -475,6 +492,7 @@ static int cdr_ratecall(struct ast_cdr *cdr)
 		break;
 	    }
 	}
+	ast_mutex_unlock(&chan->lock);
     }
 
     /*
@@ -2065,7 +2083,7 @@ int load_module(void)
     /*
      * Create worker thread to post CDR data to the Rating Engine
      */
-    if (pthread_create(&poster_thread, NULL, poster_worker, NULL)) {
+    if (ast_pthread_create(&poster_thread, NULL, poster_worker, NULL)) {
 	ast_log(LOG_ERROR, "Unable to create CDR Rating Engine Poster thread.\n");
 	return -1;
     }
